/**
 * Created by knifhen on 05/10/15.
 */
require('sa-util');
var logger = require('sa-logger');
var Q = require('q');
var PlayerDAO = function (client) {
    this.client = client;
};
module.exports = PlayerDAO;
var PLAYERS_KEY = 'Players';
PlayerDAO.prototype = {
    get: function (playerId, callback) {
        if (callback === void 0) { callback = null; }
        var deferred = Q.defer();
        var self = this;
        Q.ninvoke(self.client, "hget", [PLAYERS_KEY, playerId])
            .then(JSON.parse)
            .then(deferred.resolve)
            .catch(deferred.reject);
        if (callback) {
            deferred.promise.nodeify(callback);
        }
        return deferred.promise;
    },
    set: function (playerId, playerObject, callback) {
        if (callback === void 0) { callback = null; }
        var deferred = Q.defer();
        var self = this;
        Q.ninvoke(self.client, "hset", [PLAYERS_KEY, playerId, JSON.stringify(playerObject)])
            .then(deferred.resolve)
            .catch(deferred.reject);
        if (callback) {
            deferred.promise.nodeify(callback);
        }
        return deferred.promise;
    },
    /**
     *
     * @param playerId
     * @param modifyPlayer - function that takes playerObject as argument, and return {} with modified playerObject and other fields
     * @param callback
     */
    updateInTransaction: function (playerId, modifyPlayer, callback) {
        if (callback === void 0) { callback = null; }
        var deferred = Q.defer();
        var self = this;
        var lock = "playerLock:" + playerId;
        self.client.watch(lock);
        self.get(playerId, function (err, playerObject) {
            if (err) {
                logger.error("Failed to get player: " + playerId + ", " + err);
                return deferred.reject(err);
            }
            else if (!playerObject) {
                logger.warn("Player doesn't exist: " + playerId);
                return deferred.reject("Player doesn't exist: " + playerId);
            }
            var cb = function (result) {
                var playerObject = result.playerObject;
                var multi = self.client.multi();
                multi.set(lock, "");
                multi.hset(PLAYERS_KEY, playerId, JSON.stringify(playerObject));
                multi.del(lock);
                multi.exec(function (err, reply) {
                    if (err || reply == null) {
                        logger.error("Failed to update player: " + playerId + ", " + err);
                        return deferred.reject("Failed to update player: " + playerId);
                    }
                    else {
                        return deferred.resolve(result);
                    }
                });
            };
            modifyPlayer(playerObject, cb);
        });
        if (callback) {
            deferred.promise.nodeify(callback);
        }
        return deferred.promise;
    },
    scan: function (callback) {
        var self = this;
        var handleScanResult = function (result) {
            callback(null, result.key, result.value);
        };
        Q.ninvoke(self.client, "exists", PLAYERS_KEY)
            .then(function (result) {
            var stream;
            if (result) {
                stream = self.client.hscanStream(PLAYERS_KEY, { count: 100 });
            }
            else {
                stream = self.client.scanStream({ count: 100 });
            }
            stream.on('data', function (results) {
                for (var i = 0; i < results.length; i += 2) {
                    handleScanResult({ key: results[i], value: results[i + 1] });
                }
            });
        })
            .catch(function (err) {
            logger.error(err);
            callback(err);
        });
    },
    del: function (playerId, callback) {
        if (callback === void 0) { callback = null; }
        var deferred = Q.defer();
        var self = this;
        Q.ninvoke(self.client, "hdel", [PLAYERS_KEY, playerId])
            .then(deferred.resolve)
            .catch(deferred.reject);
        if (callback) {
            deferred.promise.nodeify(callback);
        }
        return deferred.promise;
    },
    exists: function (playerId, callback) {
        if (callback === void 0) { callback = null; }
        var deferred = Q.defer();
        var self = this;
        Q.ninvoke(self.client, "hexists", [PLAYERS_KEY, playerId])
            .then(deferred.resolve)
            .catch(deferred.reject);
        if (callback) {
            deferred.promise.nodeify(callback);
        }
        return deferred.promise;
    },
    getPlayerId: function (extIds, token_for_business) {
        if (token_for_business === void 0) { token_for_business = null; }
        var self = this;
        if (token_for_business) {
            extIds.unshift(token_for_business);
        }
        return Q.ninvoke(self.client, "hmget", ["ExtIdToPlayerId"].concat(extIds))
            .then(function (playerIds) {
            var playerId = playerIds.filter(function (e) {
                return e != null;
            }).first();
            playerId = playerId ? playerId : extIds;
            return playerId;
        });
    }
};
